
# q-expansion of newform 1260.4.a.l, downloaded from the LMFDB on 21 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1260
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [5], [7], [36], [-34], [30], [-16], [48], [126], [8], [74], [138], [-352], [396], [78], [60], [-70], [-664], [-156], [410], [344], [-444], [1002], [290], [942], [-16], [1812], [-634], [-1302], [992], [-132], [1098], [2864], [-66], [2288], [38], [3272], [-2676], [-234], [-204], [962], [-36], [3530], [-978], [4136], [-364], [6104], [1692], [-2182], [-318], [3348], [-2830], [3300], [3654], [-3912], [534], [-2296], [-1534], [9078], [6620], [-5754], [4580], [1632], [-4582], [3270], [-11380], [-1414], [-3204], [2690], [-3690], [4476], [-2656], [-4390], [-5380], [-11268], [8910], [-8002], [11694], [-214], [1476], [-994], [-1476], [818], [8192], [540], [4710], [-14614], [-18282], [-5944], [2340], [4488], [-1576], [15948], [2804], [-14388], [15894], [-14550], [-21292], [-18178], [-13384], [-23274], [-12900], [5766], [-21580], [-8494], [14148], [-16434], [-16380], [5882], [-8536], [-11206], [-23694], [29720], [6440], [-23082], [7868], [11172], [1974], [13884], [11858], [-18742], [16206], [3396], [13376], [-28386], [-6730], [-2520], [13136], [-15298], [-22732], [6432], [-6448], [22826], [-19374], [-2566], [5334], [-31228], [-25002], [5766], [26480], [1542], [-26032], [9612], [-24718], [37512], [-37738], [29406], [-23896], [-29712], [-2662], [27066], [-16792], [8748], [-31408], [21516], [37736], [40362], [48314], [11574], [-9540], [2514], [44984], [-6732], [-18342], [1044], [17120], [19910], [58466], [16782], [39540], [-28006], [-14772], [-27790], [49304], [5826], [40724], [-64770], [4352], [-8254], [31544], [54660], [23354], [-49770], [-14352], [54702], [-40414], [24956], [66602], [-46824], [-62494], [3588], [10820], [-71322], [41436], [13542], [-19678], [-56998], [-3834], [-60636], [38550], [-30400], [-24274], [36170], [40788], [24750], [47864], [-28188], [41166], [86936], [52634], [23214], [47768], [14172], [-12360], [40322], [-79360], [31026], [-94392], [-5082], [-82762], [38120], [52062], [43832], [38172], [-2794], [558], [-5172], [-70984], [76044], [-71638], [-35440], [-59824], [-28242], [-7108], [32820], [-54670], [-28818], [-20004], [66648], [31740], [-74008], [61832], [-56002], [18318], [-33840], [-97864], [-75684], [-47860], [-105792], [-21526], [-270], [-23304], [-50398], [63582], [-103836], [-6802], [17516], [37446], [-79126], [-58696], [48612], [-73294], [-5146], [134886], [-58768], [-90234], [133530], [66296], [-110466], [-16918], [-83440], [-56806], [59672], [91298], [58808], [-98700], [-50482], [-74902], [24084], [-153108], [-40768], [-18276], [-11302], [-33628], [112884], [63362], [-48594], [-15136], [-102390], [76278], [-73428], [-101550], [77508], [62186], [41190], [-81160], [-139674], [-77100], [69428], [119114], [-174282], [-46240], [-1812], [-166300], [60386], [26724], [12890], [-68124], [55274], [45324], [-143754], [29550], [-34792], [74136], [27890], [-104316], [64932], [147362], [-13098], [147776], [115730], [-85242], [32144], [-67398], [32546], [764], [-86764], [20064], [39198], [-149722], [141030], [15920], [36132], [148724], [-155076], [112574], [210558], [74546], [-135952], [-40102], [11634], [-60258], [-178504], [45462], [60420], [131594], [116408], [159264], [24534], [-46312], [85706], [-74418], [-25936], [-29770], [-169218], [55032], [-69684], [-3462], [161544], [13610], [-2766], [-47376], [-127404], [-45772], [215210], [12894], [-10384], [65786], [-106500], [-157048], [170928], [-70410], [103568], [-225574], [-170004], [240180], [86882], [82146], [219242], [63078], [147810], [-64072], [160098], [144872], [118068], [55400], [166886], [-158944], [2484], [-30334], [146478], [-84684], [4484], [233700], [122330], [-134176], [23442], [12380], [-143394], [142418], [250554], [174152], [156102], [-6114], [279128], [41258], [98550], [-13084], [-114156], [722], [-147474], [141708], [185612], [-83518], [-176298], [-20316], [-111832], [132966], [59052], [170694], [53270], [-226536], [-85644], [-10870], [137454], [156228], [-283506], [155072], [-18744]]
